//----------------------------------------------------------------------------

//:         LEVEL
//:         1.00
//:     IAR EWAVR 6.12
//: ATMega8
//E-mail:         wubblick@yahoo.com
//Fuses:          FF D5FF

//----------------------------------------------------------------------------

#include "Main.h"
#include "Usonic.h"
#include "Port.h"
#include "Commands.h"
#include "Sound.h"

//-----------------    : -----------------

// 2     .
//       ( Sound).

//------------------------------ : ---------------------------------

volatile static bool fTick;        //  watchdog-
static char tick_div;              //  
static bool tick;                  //    

//--------------------------  : ------------------------------

void main(void);                   // 
void Main_Wdt_Init(void);          // watchdog-
void Main_Rst_Wdt(bool t);         // watchdog-
void Main_Ports_Init(void);        // 
void Main_Timer_Init(void);        //  
#pragma vector = TIMER2_COMPA_vect
__interrupt void Sys_Timer(void);  //  
__monitor bool Main_GetTick(void); //  

//----------------------------------------------------------------------------
//-------------------------  : ------------------------------
//----------------------------------------------------------------------------

void main(void)
{
  Main_Wdt_Init();          // watchdog-
  Main_Ports_Init();        // 
  Main_Timer_Init();        //  
  Usonic_Init();            //  
  Port_Init();              //  RS-485
  __enable_interrupt();     // 

  while(1)                  // 
  {
    tick = Main_GetTick();  //  
    Usonic_Exe(tick);       // 
    Commands_Exe();         // 
    Port_Exe(tick);         // 
    Sound_Exe(tick);        // 
    Main_Rst_Wdt(tick);     // watchdog-
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//-------------------  watchdog-: ------------------------

void Main_Wdt_Init(void)
{
  __watchdog_reset();
  WDTCSR |= (1 << WDCE) | (1 << WDE); // watchdog-
  WDTCSR = (1 << WDE) | (1 << WDP2) | (1 << WDP0); // 500 
}

//------------------------  watchdog-: -------------------------

void Main_Rst_Wdt(bool t)
{
  if(t)                 //   , 
    __watchdog_reset(); // watchdog-
}

//------------------------  : -----------------------------

void Main_Ports_Init(void)
{
  DDRB  = I_DDRB;
  PORTB = I_PORTB;
  DDRC  = I_DDRC;
  PORTC = I_PORTC;
  DDRD  = I_DDRD;
  PORTD = I_PORTD;
}

//------------------    ------------------------

#define T2VAL (char)((F_CLK * T_SYS / 2 / 8) + 0.5) //  CK/64
//   2    T_SYS.
//          .
void Main_Timer_Init(void)
{
  TCCR2A = 1 << WGM21;  //set OC2B, CTC
  TCCR2B = 1 << CS21;   //CK/8
  OCR2A = T2VAL - 1;    //  
  TIFR2 = 1 << OCF2A;   //  
  TIMSK2 = 1 << OCIE2A; //   
  tick_div = 0;         //   
  fTick = 1;            //   
}

//--------------------   : --------------------------

bool Main_GetTick(void)
{
  if(!fTick) return(0);
  fTick = 0;
  return(1);
}

//-------------------   : -------------------------

#pragma vector = TIMER2_COMPA_vect
__interrupt void Sys_Timer(void)
{
  if(tick_div++ & 1) //   -
    fTick = 1;       //    
}

//----------------------------------------------------------------------------
