//----------------------------------------------------------------------------

//   Wake   RS-485

//----------------------------------------------------------------------------

#include "Main.h"
#include "Port.h"
#include "Wake.h"

//----------------------------- : -----------------------------------

#define BAUD  19200 // 
#define TX_DELAY 20 // , 

#define UBRRV (int)((F_CLK * 1000000.0)/(16.0 * BAUD) - 0.5) //

//------------------------------- : --------------------------------

char Rx_Sta,        //   
     Rx_Pre,        //  
     Rx_Add,        //,    
     Rx_Cmd,        // 
     Rx_Nbt,        //    
     Rx_Dat[FRAME], //  
     Rx_Crc,        //   
     Rx_Ptr;        //    

char Command;       //   

char Tx_Sta,        //   
     Tx_Pre,        //  
     Tx_Add,        //,   
     Tx_Cmd,        //,   
     Tx_Nbt,        //    
     Tx_Dat[FRAME], //   
     Tx_Crc,        //   
     Tx_Ptr;        //    

static char TxTimer; //     RS-485

//-------------------------------- : ----------------------------------

#pragma vector = USART_TX_vect
__interrupt void PC_Tx(void);    //   
#pragma vector = USART_RX_vect
__interrupt void PC_Rx(void);    //   
void Do_Crc8(char b, char *crc); //  

//------------------------  UART: -------------------------------

void Port_Init(void)
{
  UCSR0C = (1<<UCSZ01) | (1<<UCSZ00); //8- 
  UBRR0L = LO(UBRRV); //  
  UBRR0H = HI(UBRRV);
  //     ,    
  UCSR0B = (1<<RXCIE0) | (1<<TXCIE0) | (1<<RXEN0) | (1<<TXEN0);

  Rx_Add = 0;
  Tx_Add = 0;
  Rx_Sta = WAIT_FEND;     // 
  Tx_Sta = SEND_IDLE;     //   
  Command = CMD_NOP;      //   
}

//-------------------  UART   : --------------------

#pragma vector = USART_RX_vect
__interrupt void PC_Rx(void)
{
  char error_flags = (UCSR0A & (1<<FE0)); //  
  char data_byte = UDR0;              // 

  if(error_flags)                     //     
  {
    Rx_Sta = WAIT_FEND;               //  
    return;
  }

  if(data_byte == FEND)               //   ,
  {
    Rx_Pre = data_byte;               //  -,
    Rx_Crc = CRC_INIT;                // CRC,
    Rx_Sta = WAIT_ADDR;               //  ,
    Do_Crc8(data_byte, &Rx_Crc);      // CRC
    Port_LINK_0;                      //  
    return;                           //
  }

  if(Rx_Sta == WAIT_FEND)             //----->   FEND,
    return;                           // 

  char Pre = Rx_Pre;                  //  -
  Rx_Pre = data_byte;                 // -
  if(Pre == FESC)                     // -  FESC,
  {
    if(data_byte == TFESC)            //    TFESC,
      data_byte = FESC;               //    FESC
    else if(data_byte == TFEND)       //    TFEND,
           data_byte = FEND;          //    FEND
         else
         {
           Rx_Sta = WAIT_FEND;        //  
           return;
         }
  }
  else
  {
    if(data_byte == FESC)             //    FESC,  
      return;                         //  -
  }

  switch(Rx_Sta)
  {
  case WAIT_ADDR:                     //----->   
    {
      Tx_Add = 0;
      if(data_byte & 0x80)            //  7    ,   
      {
        data_byte = data_byte & 0x7F; //  7,   
        if(data_byte == 0 || data_byte == Rx_Add) //    ,
        {
          Tx_Add = data_byte;
          Rx_Sta = WAIT_CMD;          //   
          Do_Crc8(data_byte, &Rx_Crc); // CRC
          break;
        }
        Rx_Sta = WAIT_FEND;           //  ,   
        Port_LINK_1;                  //   
        break;
      }                               //  7   , 
      Rx_Sta = WAIT_CMD;              //    
    }
  case WAIT_CMD:                      //----->   
    {
      if(data_byte & 0x80)            //  7 
      {
        Rx_Sta = WAIT_FEND;           //  7   ,
        Command = CMD_ERR;            // 
        break;
      }
      Rx_Cmd = data_byte;             // 
      Rx_Sta = WAIT_NBT;              //    
      Do_Crc8(data_byte, &Rx_Crc);    // CRC
      break;
    }
  case WAIT_NBT:                      //----->    
    {
      if(data_byte > FRAME)           //   > FRAME,
      {
        Rx_Sta = WAIT_FEND;
        Command = CMD_ERR;            // 
        break;
      }
      Rx_Nbt = data_byte;
      Rx_Ptr = 0;                     //  
      Rx_Sta = WAIT_DATA;             //   
      Do_Crc8(data_byte, &Rx_Crc);    // CRC
      break;
    }
  case WAIT_DATA:                     //----->   
    {
      if(Rx_Ptr < Rx_Nbt)             //    ,
      {
        Rx_Dat[Rx_Ptr++] = data_byte; //   ,
        Do_Crc8(data_byte, &Rx_Crc);  // CRC
        break;
      }
      if(data_byte != Rx_Crc)         //   ,   CRC
      {
        Rx_Sta = WAIT_FEND;           // CRC  ,
        Command = CMD_ERR;            // 
        break;
      }
      Rx_Sta = WAIT_FEND;             //  ,
      Command = Rx_Cmd;               //   
      break;
    }
  }
}

//------------------  UART   : -------------------

#pragma vector = USART_TX_vect
__interrupt void Tx_PC(void)
{
  char data_byte;

  if(Tx_Pre == FEND)                  //  ,
  {
    data_byte = TFEND;                // TFEND  FEND
    Tx_Pre = data_byte;
    UDR0 = data_byte;
    return;
  }
  if(Tx_Pre == FESC)                  //  ,
  {
    data_byte = TFESC;                // TFESC  FESC
    Tx_Pre = data_byte;
    UDR0 = data_byte;
    return;
  }

  switch(Tx_Sta)
  {
  case SEND_ADDR:                     //----->  
    {
      if(Tx_Add)                      //    ,
      {
        data_byte = Tx_Add | 0x80;    // 7    
        Tx_Sta = SEND_CMD;
        break;
      }
      else
        Tx_Sta = SEND_CMD;            //   
    }
  case SEND_CMD:                      //----->  
    {
      data_byte = Tx_Cmd & 0x7F;
      Tx_Sta = SEND_NBT;
      break;
    }
  case SEND_NBT:                      //----->   
    {
      data_byte = Tx_Nbt;
      Tx_Sta = SEND_DATA;
      Tx_Ptr = 0;                     //    
      break;
    }
  case SEND_DATA:                     //----->  
    {
      if(Tx_Ptr < Tx_Nbt)
        data_byte = Tx_Dat[Tx_Ptr++];
      else
      {
        data_byte = Tx_Crc;           // CRC
        Tx_Sta = SEND_CRC;
      }
      break;
    }
  default:
    {
      Port_DIR_0;                     // RS-485  
      Tx_Sta = SEND_IDLE;             //  
      return;
    }
  }

  if(Tx_Sta == SEND_CMD)              // CRC
    Do_Crc8(data_byte & 0x7F, &Tx_Crc);
      else Do_Crc8(data_byte, &Tx_Crc);
  Tx_Pre = data_byte;                 // -
  if(data_byte == FEND || data_byte == FESC)
    data_byte = FESC;                 // FESC,   
  UDR0 = data_byte;
}

//---------------------    -------------------------

void Do_Crc8(char b, char *crc)
{
  for(char i = 0; i < 8; b = b >> 1, i++)
    if((b ^ *crc) & 1) *crc = ((*crc ^ 0x18) >> 1) | 0x80;
     else *crc = (*crc >> 1) & ~0x80;
}

//-------------------------   ----------------------------------

void Port_Tx_Frame(void)
{
  TxTimer = ms2sys(TX_DELAY);         // 
  Tx_Sta = SEND_FEND;                 //   
  Port_LINK_0;                        //  
}

//---------------------------  : -------------------------------

void Port_Exe(bool t)
{
  if(t)
  {
    char a = 0;
    if(!Pin_ADDR1) a |= 0x01;         // 
    if(!Pin_ADDR2) a |= 0x02;
    if(!Pin_ADDR3) a |= 0x04;
    if(!Pin_ADDR4) a |= 0x08;
    if(!Pin_ADDR5) a |= 0x10;
    if(!Pin_ADDR6) a |= 0x20;
    if(!Pin_ADDR7) a |= 0x40;
    Rx_Add = a;

    if(Tx_Sta == SEND_FEND)           //   
    {
      if(TxTimer == 0)                //   
      {
        char data_byte = FEND;
        Tx_Crc = CRC_INIT;            // CRC,
        Do_Crc8(data_byte, &Tx_Crc);  // CRC
        Tx_Sta = SEND_ADDR;           //   
        Tx_Pre = TFEND;
        Port_DIR_1;                   // RS-485  
        UDR0 = data_byte;             // FEND
        Port_LINK_1;                  //   
      }
      else TxTimer--;                 //  
    }
  }
}

//----------------------------------------------------------------------------
